{%extends "admin/change_form.html" %}

{%block after_related_objects%}
{%if original.pk and original.deleted_flag%}
  <input type="submit" name="undelete" value="Undelete these objects" />
{%endif%}
{%endblock%}
