{%extends "softdelete/base.html"%}

{%block content%}

<form method="POST">
  {%csrf_token%}
  <table>
    {{form.as_table}}
    <tr>
      <td><input type="submit" name="action" value="Undelete"></td>
      <td><input type="submit" name="action" value="Create"></td>
    </tr>
  </table>
</form>


{%endblock%}
